#!/usr/bin/env python

import requests
from requests import RequestException
import json
from . import exceptions

DEMO_DATA_URL = 'http://api.demo.360telemetry.com/api/v1/service/json'


class API:
    """
    Handles the dispatch of snapshots to the server.
    """

    def __init__(self, device_key, unit_identifier, service_url=DEMO_DATA_URL):
        """
        Create a new instance.
        :param device_key:
        :param unit_identifier:
        :param service_url:
        :return:
        """
        self.device_key = device_key
        self.unit_identifier = unit_identifier
        self.service_url = service_url
        self.snapshots = []

    def add(self, new_snapshot):
        """
        Add a snapshot, ready for dispatch.
        :param new_snapshot:
        :return:
        """
        self.snapshots.append(new_snapshot)

    def dispatch(self):
        """
        Dispatch currently-stored snapshots to the server.
        :raises ApiException:
        :return:
        """

        response = None

        try:
            response = requests.post(
                self.service_url,
                data=self.__build_json(),
                headers={'X-Api-Key': self.device_key}
            )

            # Did we receive any configuration?
            return response.json()

        except ValueError as exception:
            # Problems deserialising the JSON response
            if response:
                raise exceptions.ApiException(exception.message)
            else:
                raise exceptions.ApiException(response.text)
        except RequestException as exception:
            # Problems communicating with the server
            raise exceptions.ApiException(exception.message[0])

    def __build_json(self):
        """
        Build a JSON string from the current state of this client
        :return:
        """
        return json.dumps(
            {
                "data": {
                    self.unit_identifier: map(lambda snapshot: snapshot.get_api_data(), self.snapshots)
                }
            }
        )



#!/usr/bin/env python


class ApiException(Exception):
    """
    Represents an exception encountered while communicating with the API.
    """

    def __init__(self, message, error_number=-1):
        self.message = message
        self.error_number = error_number



#!/usr/bin/env python

import time


class Snapshot(object):
    """A snapshot of data ready to send to the API"""

    def __init__(self, timestamp=None):
        """
        Creates a new snapshot for the specified timestamp. If a timestamp is not provided, it is assumed to be a
        snapshot taken at the current time.
        :param timestamp:
        :return:
        """
        self.timestamp = timestamp if timestamp else time.time()
        self.channels = {}
        self.alarms = {}

    def set_channel(self, channel_id, value):
        """
        Set a channel ID to a specified value for this snapshot.
        :param channel_id:
        :param value:
        :return:
        """
        self.channels[channel_id] = value

    def get_channel(self, channel_id):
        """
        Get a channel ID from this snapshot.
        :param channel_id:
        :return:
        """
        return self.channels.get(channel_id, None)

    def set_alarm(self, channel_id, alarm):
        """
        Set an alarm on a channel.
        :param channel_id:
        :param alarm:
        :return:
        """
        self.alarms[channel_id] = alarm

    def get_api_data(self):
        """
        Build an API representation of this snapshot.
        :return:
        """
        data = {}
        for channel_id, channel_value in self.channels.iteritems():
            data[channel_id] = [channel_value]
            if channel_id in self.alarms:
                data[channel_id].append(self.alarms.get(channel_id, ''))

        return {'timestamp': self.timestamp, 'data': data}




from setuptools import setup

setup(
    name='mym2m-client',
    version='0.1.0',
    packages=['mym2m'],
    install_requires=[
      'requests',
    ],
    license='MIT',
    author='360 Telemetry',
    author_email='info@360telemetry.com',
    long_description=open('README.md').read(),
    description='Python SDK for the MyM2M Web JSON Device API',
    url='https://bitbucket.org/360telemetry/mym2m-python-sdk'
)
